#ifndef __IUF_TRIANGLE
#define __IUF_TRIANGLE
#include <math.h>
#include <vcg/Point3.h>
#include <vector>
using namespace std;
using namespace vcg;

/** Trivial class to represent a vertex with its normal and color.
Version History {\small 
\begin{itemize}
\item 0.4 First Release.
\end{itemize}
}
*/

class Vertex
{
	public:
/// The Vertex coords
		Point3f v;
/// The Vertex Normal
		Point3f n;
/// The Vertex Color
		Point3f c;
};

/** Trivial class to represent a triangle with its normal.
For each triangle we store three vertex with normals and colors. 
Note that this is a very naive approach, we do not take into account
that vertexes can be shared.

Version History {\small 
\begin{itemize}
\item 0.2 First Release.
\item 0.2s
Added per vertex color and normals.
Added Trivial constructor and from three points.
\item 0.4 Changed internal storing of coords, color and normals, now 
a triangle is just three vertex
\end{itemize}
}
*/
class Triangle
{
public:
	/// Default constructor
	Triangle(){}
	/** Three Vertex constructor. 
	It computes also the normals.
	*/
	Triangle(Point3f const &v0,Point3f const &v1,Point3f const &v2){
		v[0].v=v0;v[1].v=v1;v[2].v=v2;
		CalcNormal();
	}
/// The three vertexes
 Vertex v[3];

/** Compute the normal of the face starting from the three vertexes
 and store it in the \Ref{n[]} member */
 void CalcNormal();

/** Flip the triangle.
And then recalculate the new (flat) normals
*/
 void Flip();

 /// Less operator needed for stl containers
 inline bool operator <  ( Triangle const & p ) const {
			return	(v[2].v!=p.v[2].v)?(v[2].v<p.v[2].v):
							(v[1].v!=p.v[1].v)?(v[1].v<p.v[1].v):
							(v[0].v<p.v[0].v);
	}
/// Equal operator needed for some stl containers
 inline bool operator == ( Triangle const & p ) const {
			return (v[0].v==p.v[0].v && v[1].v==p.v[1].v && v[2].v==p.v[2].v);
	} 

};

/** Trivial class to represent a simple triangular mesh.

Version History {\small 
\begin{itemize}
\item 0.2  First Release.
\item 0.2s Added Color mesh loading and drawing, smooth normals computation and resizing.
\item 0.4  Added Normal Mesh Loading, Display lists, Vertex arrays, 
\item 0.5  Added texture name member, texture loading function and simple xy planar mapping ot texture coordinates.
\end{itemize}
}
*/
class Mesh
{
public:
	/** standard constructor. By default the mesh has no color. */
	Mesh(){has_color=false;dl=0;ti=0;}
  
	/** destructor should only de-allocate opengl display lists. */
	~Mesh(){if(dl) glDeleteLists(dl,1);}

  /** All the triangles are stored in a STL vector. 
	@name vector<\Ref{Triangle}> T*/
	vector<Triangle> T;
	
  /// The Index of the Display List of the Mesh
	int dl;

  /// The name of the texture
  GLuint ti;

	/** Simply Draw all the Triangles stored in \Ref{T}.
	If the mesh has_color then the defined per-vertex colors
	are used, else the current color is used. If the mesh has a texture 
	a simple planar mapping onto the xy plane is used to setup texture 
	coords . 
	*/
	int Draw();
	
	/** Create the display list for this Mesh.
	*/
	void GenerateDL();
  /** Draw the mesh using Display lists.
	If the list was never generated it is generated on the fly
	*/
	int DrawDL();

	/** Draw the mesh using Vertex arrays.
	*/
	int DrawA();

	/** Load a mesh from a file.
	The format is the so-called {\em RAW} ascii format, for each triangle
	just the ascii print of xyz coordinates of its vertexes. 
	E.g. a tetrahedron is stored as:
	\begin{verbatim}
	 1  1  1    -1 -1  1     1 -1 -1
	-1 -1  1     1  1  1    -1  1 -1
	 1 -1 -1    -1 -1  1    -1  1 -1
	-1  1 -1     1 -1 -1     1  1  1
	\end{verbatim}

	The Open function understand the type of the file from the extension.
	It manages the standard ".raw" and the new "*.rawc" (mesh with per vertex color). 
	The loading of a "rawc" is done by the \Ref{OpenColor} function
*/
	void Open(char *filename);
  
/// true if the loaded mesh has color
	bool has_color;

/** Load a texture from file.
The texture must be in PNG format and have standard 2^n x 2^m size.
The texture name assigned to this texure is stored in the class member ti.
*/
 void OpenTexture(char *filename);

/** Load a mesh with color (rawc) from file.
	In the extended raw (*.rawc) format for each 
	face we also have three colors.
	The three colors (in the 0-1 range) are written after the three coords. 
	E.g. a Red-Blue tetrahedron can be specified as follows:

	\begin{verbatim}
	 1  1  1    -1 -1  1     1 -1 -1    1 0 0  1 0 0  0 0 1
	-1 -1  1     1  1  1    -1  1 -1    1 0 0  1 0 0  0 0 1
	 1 -1 -1    -1 -1  1    -1  1 -1    0 0 1  1 0 0  0 0 1
	-1  1 -1     1 -1 -1     1  1  1    0 0 1  0 0 1  1 0 0
	\end{verbatim}
*/
	void OpenColor(char *filename);

/** Load a mesh with normal (rawn) from file.
	In the extended raw (*.rawc) format for each 
	face we also have three normals.
	The three normals(in the 0-1 range) are written immediately after the each vertex coords 
	(warning different from color format!). 
*/
	void OpenNormal(char *filename);

/** Calculate smooth per vertex normal.
The normals are calculated by averaging the normals of all the faces
incident on a same vertex. Obviously we should be sure that all 
the face has the same vertex ordering (otherwise we could average 
opposite normals).
This can be efficiently done by exploiting the 
map associative container of STL. 
A map<key,data> is a Associative Container that 
associates objects of type Key with objects of type Data.
The easiest way to think a map is a vector indexed by object of type key instead of integers.
In our case we use a map<point,normal>. We do all in three steps:
\begin{itemize}
\item In the first step we clear all the entry, 
\item then we accumalte all the vertex normals and 
\item in the third pass we normalize the accumulated normals, and assign them to their vertexes.
\end{itemize}
*/
	void Smooth();

/** Resize a mesh so that it can fit in the unitary box. 
It is useful when loading a mesh of unknown size. 
An object can also be scaled to a user-defined size by using the {\em size} param.
*/
	void Normalize(float size=1);
/** Flip orientaion of all the triangles.
*/
void Flip();

};
#endif